@@ -12,3 +12,13 @@ rgl.setWheelCallback <- function(rotate=NULL, dev = cur3d(), subscene = currentS
 
 rgl.getWheelCallback <- function(dev = cur3d(), subscene = currentSubscene3d(dev)) 
     .Call(rgl_getWheelCallback, as.integer(dev), as.integer(subscene))
+
+rgl.setAxisCallback <- function(axis, draw = NULL, dev = cur3d(), subscene = currentSubscene3d(dev)) {
+  stopifnot(length(axis) == 1, axis %in% 1:3)
+  .Call(rgl_setAxisCallback, draw, as.integer(dev), as.integer(subscene), as.integer(axis - 1))
+  invisible(NULL)
+}
+
+rgl.getAxisCallback <- function(axis, dev = cur3d(), subscene = currentSubscene3d(dev)) {
+  .Call(rgl_getAxisCallback, as.integer(dev), as.integer(subscene), as.integer(axis - 1))
+}
