@@ -7,7 +7,7 @@ plot3d.default <- function(x, y = NULL, z = NULL,
         lwd = material3d("lwd"),
         radius = avgscale*size/60, add = FALSE, aspect = !add, 
         xlim = NULL, ylim = NULL, zlim = NULL,
-        forceClipregion = FALSE, ...) {
+        forceClipregion = FALSE, decorate = !add, ...) {
     if (!add) next3d()
     skip <- par3d(skipRedraw=TRUE)
     on.exit(par3d(skip))
@@ -62,11 +62,12 @@ plot3d.default <- function(x, y = NULL, z = NULL,
                                  rep(range(z, na.rm=TRUE), c(2,2))))
 	)
     useSubscene3d(savesubscene)
-    if (!add) {
-    	result <- c(result, decorate3d(xlab=xlab, ylab=ylab, zlab=zlab, aspect = aspect, 
-                                       xlim=xlim, ylim=ylim, zlim=zlim, ...))
+    if (decorate)
+      result <- c(result, decorate3d(xlab=xlab, ylab=ylab, zlab=zlab, aspect = aspect, 
+                               xlim=xlim, ylim=ylim, zlim=zlim, ...))
+    if (!add)
     	highlevel(result)
-    } else 
+    else 
     	lowlevel(result)
 }
 
@@ -96,7 +97,7 @@ plot3d.mesh3d <- function(x, xlab = "x", ylab = "y", zlab = "z", type = c("shade
 decorate3d <- function(xlim = NULL, ylim = NULL, zlim = NULL, 
                        xlab = "x", ylab = "y", zlab = "z", 
                        box = TRUE, axes = TRUE, main = NULL, sub = NULL,
-                       top = TRUE, aspect = FALSE, expand = 1.03, ...) {
+                       top = TRUE, aspect = FALSE, expand = 1.03, tag = material3d("tag"), ...) {
   
   if (is.logical(aspect)) {
     autoscale <- aspect
@@ -113,14 +114,14 @@ decorate3d <- function(xlim = NULL, ylim = NULL, zlim = NULL,
     if (is.null(zlim))
       zlim <- ranges$zlim
     ind <- c(1,1,2,2)
-    result <- c(result, strut=segments3d(xlim[ind], ylim[ind], zlim[ind]))
+    result <- c(result, strut=segments3d(xlim[ind], ylim[ind], zlim[ind], tag = tag))
   }
   
   if (autoscale) aspect3d(aspect)
   
-  if (axes) result <- c(result, axes=axes3d(box=box, expand=expand))
+  if (axes) result <- c(result, axes=axes3d(box=box, expand=expand, tag = tag))
   result <- c(result, title3d(xlab = xlab, ylab = ylab, zlab = zlab, 
-                              main = main, sub = sub))
+                              main = main, sub = sub, tag = tag))
   
   if (top) rgl.bringtotop()
   
