@@ -473,6 +473,8 @@
           fat_lines = this.isSet(flags, this.f_fat_lines),
           is_twosided = this.isSet(flags, this.f_is_twosided),
           has_fog = this.isSet(flags, this.f_has_fog),
+          has_normals = (typeof obj.normals !== "undefined") ||
+                        obj.type === "sphere",
           gl = this.gl || this.initGL(),
           count,
           pass, mode, pmode,
@@ -526,13 +528,13 @@
         enabled.normLoc = this.doNormals(obj);
 
       if (fixed_size) {
-        gl.uniform2f( obj.textScaleLoc, 0.75/this.vp.width, 0.75/this.vp.height);
+        gl.uniform3f( obj.textScaleLoc, 0.75/this.vp.width, 0.75/this.vp.height, 1.0);
       }
       
       if (fixed_quads) {
         gl.enableVertexAttribArray( obj.ofsLoc );
         enabled.ofsLoc = true;
-        gl.vertexAttribPointer(obj.ofsLoc, 2, gl.FLOAT, false, 4*obj.vOffsets.stride, 4*obj.vOffsets.oofs);
+        gl.vertexAttribPointer(obj.ofsLoc, 3, gl.FLOAT, false, 4*obj.vOffsets.stride, 4*obj.vOffsets.oofs);
       }
 
       for (pass = 0; pass < obj.passes; pass++) {
@@ -542,8 +544,12 @@
 
       	mode = fat_lines && (is_lines || pmode === "lines") ? "TRIANGLES" : this.mode4type[type];
 
-      	if (is_twosided)
+      	if (is_twosided) {
       	  gl.uniform1i(obj.frontLoc, pass !== 0);
+      	  if (has_normals) {
+      	    gl.uniformMatrix4fv(obj.invPrMatLoc, false, new Float32Array(this.invPrMatrix.getAsArray()));
+      	  }
+      	}
 
         gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, obj.ibuf[pass]);
         if (!this.opaquePass) {
@@ -695,6 +701,7 @@
                              obj.values[baseofs+2]);
         sphereMV.multRight(saveMV);
         this.mvMatrix = sphereMV;
+        this.setnormMatrix2();
         this.setprmvMatrix();
         if (drawing) {
           if (nc > 1) {
@@ -787,7 +794,7 @@
       
       var norigs = obj.vertices.length,
           savenorm = new CanvasMatrix4(this.normMatrix),
-          iOrig;
+          iOrig, adj, offset;
 
       this.normMatrix = subscene.spriteNormmat;
       userMatrix = obj.userMatrix;
@@ -801,6 +808,8 @@
       if (this.prmvMatrix !== null)
          origPRMV = new CanvasMatrix4( this.prmvMatrix );
 
+      offset = obj.offset;
+        
       for (iOrig=0; iOrig < norigs; iOrig++) {
         if (this.opaquePass)
           j = iOrig;
@@ -811,6 +820,8 @@
                           origMV);
         radius = obj.radii.length > 1 ? obj.radii[j][0] : obj.radii[0][0];
         this.mvMatrix = new CanvasMatrix4(userMatrix);
+        adj = this.getAdj(obj, j, offset);
+        this.mvMatrix.translate(1 - 2*adj[0], 1 - 2*adj[1], 1 - 2*adj[2]);
         this.mvMatrix.scale(radius);
         this.mvMatrix.translate(pos[0]/pos[3], pos[1]/pos[3], pos[2]/pos[3]);
         this.setprmvMatrix();
@@ -880,7 +891,7 @@
 
       if (this.opaquePass) {
         context = context.slice();
-        context.push(obj.cube.id);
+        context.push(obj.id);
       } 
       
       drawing = this.opaquePass !== is_transparent;
@@ -988,7 +999,7 @@
     rglwidgetClass.prototype.drawBackground = function(id, subsceneid) {
       var gl = this.gl || this.initGL(),
           obj = this.getObj(id),
-          bg, i, savepr, savemv;
+          bg, i, savepr, saveinvpr, savemv;
 
       if (!obj.initialized)
         this.initObj(obj);
@@ -1008,8 +1019,10 @@
       this.fogScale = obj.fogscale;
       if (typeof obj.quad !== "undefined") {
         savepr = this.prMatrix;
+        saveinvpr = this.invPrMatrix;
         savemv = this.mvMatrix;
         this.prMatrix = new CanvasMatrix4();
+        this.invPrMatrix = new CanvasMatrix4();
         this.mvMatrix = new CanvasMatrix4();
         gl.disable(gl.BLEND);
         gl.disable(gl.DEPTH_TEST);
@@ -1017,6 +1030,7 @@
         for (i=0; i < obj.quad.length; i++)
           this.drawObjId(obj.quad[i], subsceneid);
         this.prMatrix = savepr;
+        this.invPrMatrix = saveinvpr;
         this.mvMatrix = savemv;
       }
     };
@@ -1058,18 +1072,18 @@
       this.setViewport(subsceneid);
 
       this.setprMatrix(subsceneid);
+      this.setInvPrMatrix();
       this.setmvMatrix(subsceneid);
+      this.setnormMatrix2();
         
       if (typeof sub.backgroundId !== "undefined" && this.opaquePass)
         this.drawBackground(sub.backgroundId, subsceneid);
 
       if (subids.length) {
-        if (subscene_has_faces) {
-          this.setnormMatrix(subsceneid);
-          if (this.isSet(sub.flags, this.f_sprites_3d) &&
-              typeof sub.spriteNormmat === "undefined") {
-            sub.spriteNormmat = new CanvasMatrix4(this.normMatrix);
-          }
+        if (subscene_has_faces &&
+            this.isSet(sub.flags, this.f_sprites_3d) &&
+            typeof sub.spriteNormmat === "undefined") {
+          sub.spriteNormmat = new CanvasMatrix4(this.normMatrix);
         }
 
         if (subscene_needs_sorting)
@@ -1123,6 +1137,9 @@
           case "spheres":
             // this.initSphereFromObj(obj);  // FIXME:  not needed?
             break;
+          case "bboxdeco":
+            result = result.concat(context.pop());
+            break;
           default:
             console.error("bad type '", type, "' in setContext");
         }
