@@ -1,9 +1,21 @@
-// These functions order the centers of displayed objects so they
-// can be drawn using the painters algorithm, necessary to support
-// transparency.  
+/**
+ * Methods related to drawing transparent objects
+ * @name ___METHODS_FOR_TRANSPARENCY___
+ * @memberof rglwidgetClass
+ * @kind function
+ * @instance
 
-// Note that objid is not obj.id when drawing spheres.
+ * These functions order the centers of displayed objects so they
+ * can be drawn using the painters algorithm, necessary to support
+ * transparency.  
 
+ * Note that objid is not obj.id when drawing spheres.
+ */
+
+/**
+ * Break objects into pieces
+ * @returns { array } Array of pieces
+ */
     rglwidgetClass.prototype.getPieces = function(context, objid, subid, obj) {
       var n = obj.centers.length,
           depth,
@@ -30,6 +42,13 @@
       return result;    
     };
     
+    /**
+     * Get pieces from sphere
+     * @returns { object }
+     * @param { array } context - 
+     * @param { numeric } subid - subscene
+     * @param { object } obj - spheres object
+     */
     rglwidgetClass.prototype.getSpherePieces = function(context, subid, obj)
     {
       if (obj.fastTransparency) 
@@ -41,6 +60,23 @@
         return this.getPieces(context, obj.id, subid, this.sphere);
     };
     
+   /**
+     * Get pieces from cube
+     * @returns { object }
+     * @param { array } context - 
+     * @param { numeric } subid - subscene
+     * @param { object } obj - spheres object
+     */
+    rglwidgetClass.prototype.getCubePieces = function(context, obj)
+    {
+      return this.getPieces(context, obj.id, 0, this.cube);
+    };
+    
+    /**
+     * Merge pieces that can be drawn in one call
+     * @returns { object }
+     * @param { array } pieces - The pieces to merge
+     */
     rglwidgetClass.prototype.mergePieces = function(pieces) {
       var result = [];
       if (pieces.length > 0) {
@@ -69,6 +105,11 @@
       return result;
     };
 
+    /**
+     * Sort pieces by depth
+     * @returns { array }
+     * @param { array } pieces - array of pieces 
+     */
     rglwidgetClass.prototype.sortPieces = function(pieces) {
       var compare = function(i,j) {
         var diff = j.depth - i.depth;
