@@ -26,8 +26,9 @@
           fat_lines = this.isSet(flags, this.f_fat_lines),
           is_brush = this.isSet(flags, this.f_is_brush),
           has_fog = this.isSet(flags, this.f_has_fog),
-          has_normals = typeof obj.normals !== "undefined",
-          needs_vnormal = (is_lit && !fixed_quads && !is_brush) || (is_twosided && (has_normals || obj.type === "spheres")),
+          has_normals = (typeof obj.normals !== "undefined") ||
+                        obj.type === "spheres",
+          needs_vnormal = (is_lit && !fixed_quads && !is_brush) || (is_twosided && has_normals),
           result;
 
       if (type === "clipplanes" || sprites_3d) return;
@@ -52,21 +53,22 @@
       if (needs_vnormal)
         result = result + "  attribute vec3 aNorm;\n"+
                           "  uniform mat4 normMatrix;\n"+
-                          "  varying vec3 vNormal;\n";
+                          "  varying vec4 vNormal;\n";
 
       if (has_texture || type === "text")
         result = result + "  attribute vec2 aTexcoord;\n"+
                           "  varying vec2 vTexcoord;\n";
 
       if (fixed_size)
-        result = result + "  uniform vec2 textScale;\n";
+        result = result + "  uniform vec3 textScale;\n";
 
       if (fixed_quads)
-        result = result + "  attribute vec2 aOfs;\n";
+        result = result + "  attribute vec3 aOfs;\n";
 
       if (is_twosided)
-        if (has_normals || obj.type === "spheres")
-          result = result + "  varying float normz;\n";
+        if (has_normals)
+          result = result + "  varying float normz;\n"+
+                            "  uniform mat4 invPrMatrix;\n";
         else
           result = result + "  attribute vec3 aPos1;\n"+
                             "  attribute vec3 aPos2;\n"+
@@ -96,25 +98,27 @@
 
       result = result + "    vCol = aCol;\n";
 
-      if (needs_vnormal)
-        result = result + "    vNormal = normalize((normMatrix * vec4(aNorm, 1.)).xyz);\n";
+      if (needs_vnormal) /* Need to normalize the xyz part */
+        result = result + "    vNormal = normMatrix * vec4(-aNorm, dot(aNorm, aPos));\n"+
+                          "    vNormal = vec4(normalize(vNormal.xyz), 1);\n";
 
       if (has_texture || type === "text")
         result = result + "    vTexcoord = aTexcoord;\n";
 
       if (fixed_size)
         result = result + "    vec4 pos = prMatrix * mvMatrix * vec4(aPos, 1.);\n"+
-                          "   pos = pos/pos.w;\n"+
-                          "   gl_Position = pos + vec4(aOfs*textScale, 0.,0.);\n";
+                          "    pos = pos/pos.w;\n"+
+                          "    gl_Position = pos + vec4(aOfs*textScale, 0.);\n";
 
       if (type === "sprites" && !fixed_size)
         result = result + "    vec4 pos = mvMatrix * vec4(aPos, 1.);\n"+
-                          "   pos = pos/pos.w + vec4(aOfs, 0., 0.);\n"+
-                          "   gl_Position = prMatrix*pos;\n";
+                          "    pos = pos/pos.w + vec4(aOfs,  0.);\n"+
+                          "    gl_Position = prMatrix*pos;\n";
 
       if (is_twosided)
-        if (has_normals || obj.type === "spheres")
-          result = result + "   normz = vNormal.z;";
+        if (has_normals)
+          /* normz should be calculated *after* projection */
+          result = result + "    normz = (invPrMatrix*vNormal).z;\n";
         else
           result = result + "   vec4 pos1 = prMatrix*(mvMatrix*vec4(aPos1, 1.));\n"+
                             "   pos1 = pos1/pos1.w - gl_Position/gl_Position.w;\n"+
@@ -204,7 +208,7 @@
                           "  uniform vec4 uFogParms;\n";
 
       if (is_lit && !fixed_quads)
-        result = result + "  varying vec3 vNormal;\n";
+        result = result + "  varying vec4 vNormal;\n";
 
       for (i = 0; i < nclipplanes; i++)
         result = result + "  uniform vec4 vClipplane"+i+";\n";
@@ -267,7 +271,7 @@
       if (fixed_quads) {
         result = result +   "    vec3 n = vec3(0., 0., 1.);\n";
       } else if (is_lit) {
-      	result = result +   "    vec3 n = normalize(vNormal);\n";
+      	result = result +   "    vec3 n = normalize(vNormal.xyz);\n";
       }
 
       if (is_twosided) {
