@@ -46,7 +46,7 @@ static void StringToRGB8(const char* string, u8* colorptr) {
   
       u8 component;
 
-      component = (HexCharToNibble(c) << 4);
+      component = static_cast<u8>(HexCharToNibble(c) << 4);
       
       if ( (c = *strptr++) == '\0')
         break;
@@ -117,12 +117,16 @@ void Color::set3iv(int* color)
 
 void Color::useClearColor() const
 {
+#ifndef RGL_NO_OPENGL
   glClearColor(data[0],data[1],data[2], data[3]);
+#endif
 }
 
 void Color::useColor() const
 {
+#ifndef RGL_NO_OPENGL
   glColor4fv(data);
+#endif
 }
 
 
@@ -229,12 +233,16 @@ bool ColorArray::hasAlpha() const
 
 void ColorArray::useArray() const
 {
+#ifndef RGL_NO_OPENGL
   glColorPointer(4, GL_UNSIGNED_BYTE, 0, (const GLvoid*) arrayptr );
+#endif
 }
 
 void ColorArray::useColor(int index) const
 {
+#ifndef RGL_NO_OPENGL  
   glColor4ubv( (const GLubyte*) &arrayptr[ index * 4] );
+#endif
 }
 
 Color ColorArray::getColor(int index) const
